/*
 * This file is part of tela the Tensor Language.
 * Copyright (c) 1994-1995 Pekka Janhunen
 */

#ifdef __GNUC__
#  pragma interface
#endif
#include <strstream.h>
#include <setjmp.h>

extern strstream err;

#define COMPTYPE Tint
#define DEFAULT_COMPVALUE 0
#define LINEARLIST TIntLL
#  include "templ/tLL.H"
#undef LINEARLIST
#undef DEFAULT_COMPVALUE
#undef COMPTYPE

class TIntStack {		// Program counter stack, common for all functions
 private:
	TIntLL LL;			// Implemented as a linear list of integers
 public:
	TIntStack() : LL(0) {}			// Initially, the stack is empty
	void push(Tint i) {LL.append(i);}
	Tint top() const {return LL[LL.length()-1];}			// stack.top() gives ref to last int pushed
	Tint top(int nth) const {return LL[LL.length()-nth];}	// stack.top(n) gives the nth-last int pushed
	Tint pop() {int L; LL=L=LL.length()-1; return LL[L];}	// stack.pop() gives stack.top() and removes it
	int length() const {return LL.length();}				// stack.length() == number of items in the stack
	Tint& operator[](int i) {return LL[LL.length()-1-i];}	// stack[0] == &stack.top(), etc.
};

extern TIntStack thePCstack;		// General program counter stack

class TJumpBuffer {
 private:
	jmp_buf jmp;
	int PCstacklen;
 public:
	TJumpBuffer() {PCstacklen=0;}
#	if defined(__GNUC__) || defined(IRIS4)
	// This should probably ALWAYS be inlined, in order to ohave setjmp to store
	// the right context... but some compilers will not accept this.
	// If you modify machine list, modify also in error.C.
	int SetJmp() {PCstacklen=thePCstack.length(); int i=setjmp(jmp); return i;}
#	else
	// Otherwise we have to use external linkage and hope it works...
	int SetJmp();
#	endif
	void LongJmp(int val) {while (thePCstack.length() > PCstacklen) thePCstack.pop(); longjmp(jmp,val);}
	~TJumpBuffer() {}
};

extern TJumpBuffer jmp1,jmp2;

extern void error();
