// tLL.H must have been included before

int LINEARLIST::oversize = 10;
int LINEARLIST::mingrowchunk = 50;
int LINEARLIST::maxgrowchunk = 1000;


LINEARLIST::LINEARLIST(int L) {
	len = L;
	alloc_len = L + oversize;
	ptr = new COMPTYPE [alloc_len];
	for (int i=0; i<L; i++) ptr[i] = DEFAULT_COMPVALUE;
}

LINEARLIST::LINEARLIST(int L, const COMPTYPE table[]) {
	len = L;
	alloc_len = L + oversize;
	ptr = new COMPTYPE [alloc_len];
	for (int i=0; i<L; i++) ptr[i] = table[i];
}

LINEARLIST::LINEARLIST(const LINEARLIST& L) {
	len = L.len;
	alloc_len = len + oversize;
	ptr = new COMPTYPE [alloc_len];
	for (int i=0; i<len; i++) ptr[i] = L.ptr[i];
}

LINEARLIST& LINEARLIST::operator=(const LINEARLIST& L) {
	int i;
	if (len < L.len) {	// doesn't fit
		delete [] ptr;
		alloc_len = L.len + oversize;
		ptr = new COMPTYPE [alloc_len];
	}
	for (i=0; i<L.len; i++) ptr[i] = L.ptr[i];
	len = L.len;
	return *this;
}

LINEARLIST& LINEARLIST::operator=(int L) {
	if (len < L) {	// doesn't fit
		COMPTYPE* oldptr = ptr;
		alloc_len = L + oversize;
		ptr = new COMPTYPE [alloc_len];
		for (int i=0; i<len; i++) ptr[i] = oldptr[i];
		delete [] oldptr;
	}
	len = L;
	return *this;
}

void LINEARLIST::grow() {
	int old_alloc_len = alloc_len;
	alloc_len = min(max(5*old_alloc_len/4,mingrowchunk),maxgrowchunk);
	COMPTYPE *newptr = new COMPTYPE [alloc_len];
	for (int i=0; i<old_alloc_len; i++) newptr[i] = ptr[i];
	delete [] ptr;
	ptr = newptr;
}

void LINEARLIST::append(const COMPTYPE& c) {
	if (alloc_len <= len) grow();
	ptr[len++] = c;
}

ostream& operator<<(ostream& o, const LINEARLIST& LL) {
	o << '[';
	for (int i=0; i<LL.length()-1; i++) o << LL[i] << ',';
	o << LL[LL.length()-1] << ']';
	return o;
}

void LINEARLIST::rawasciioutput(ostream& o) const {
	for (int i=0; i<length()-1; i++) o << (*this)[i] << ',';
	o << (*this)[length()-1];
}

void LINEARLIST::rawasciiinput(istream& i) {
	len = 0;
	COMPTYPE c;
	char ch;
	for (;;) {
		i >> c;
		append(c);
		i >> ch;
		if (ch != ',') {i.putback(ch); break;}
	}
}

