/*
 * tbinop.C - 'template' for a binary operator, included separately for +,-,*,/
 */

void OPFNAME(Tobject& c, const Tobject& a, const Tobject& b) {
	int i,N;
	Tint I; Treal X; Tcomplex Z;
	switch (pair(a.kind(),b.kind())) {
	case pair(Kint,Kint):
		c = OP(a.IntValue(),b.IntValue());
		gl_nops++;
		break;
	case pair(Kint,Kreal):
		c = OP(Treal(a.IntValue()),b.RealValue());
		gl_nops++;
		break;
	case pair(Kint,Kcomplex):
		c = OP(Tcomplex(a.IntValue()),b.ComplexValue());
		gl_nops++;
		break;
	case pair(Kint,KIntArray):
		{
			c.copydimsIntArray(b);
			I = a.IntValue();
			N = b.dims().length();
			Tint *cp=c.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(I,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kint,KRealArray):
		{
			c.copydimsRealArray(b);
			X = Treal(a.IntValue());
			N = b.dims().length();
			Treal *cp=c.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kint,KComplexArray):
		{
			c.copydimsComplexArray(b);
			Z = Tcomplex(Treal(a.IntValue()),0.0);
			N = b.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,Kint):
		c = OP(a.RealValue(),Treal(b.IntValue()));
		gl_nops++;
		break;
	case pair(Kreal,Kreal):
		c = OP(a.RealValue(),b.RealValue());
		gl_nops++;
		break;
	case pair(Kreal,Kcomplex):
		c = OP(Tcomplex(a.RealValue()),b.ComplexValue());
		gl_nops++;
		break;
	case pair(Kreal,KIntArray):
		N = b.dims().length();
		X = a.RealValue();
		if (&c == &b) {
			Tobject c1;
			c1.copydimsRealArray(b);
			Treal *c1p=c1.RealPtr(); Tint *bp=b.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(X,bp[i]);
			c = c1;
		} else {
			c.copydimsRealArray(b);	// make c a real array of the same dims as b
			Treal *cp=c.RealPtr(); Tint *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
		}
		gl_nops+= N;
		break;
	case pair(Kreal,KRealArray):
		{
			c.copydimsRealArray(b);
			N = b.dims().length();
			X = a.RealValue();
			Treal *cp=c.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,KComplexArray):
		{
			c.copydimsComplexArray(b);
			Z = a.RealValue();
			N = b.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,Kint):
		c = OP(a.ComplexValue(),Tcomplex(b.IntValue()));
		gl_nops++;
		break;
	case pair(Kcomplex,Kreal):
		c = OP(a.ComplexValue(),Tcomplex(b.RealValue()));
		gl_nops++;
		break;
	case pair(Kcomplex,Kcomplex):
		c = OP(a.ComplexValue(),b.ComplexValue());
		gl_nops++;
		break;
	case pair(Kcomplex,KIntArray):
		N = b.dims().length();
		Z = a.z;
		if (&c == &b) {
			Tobject c1;
			c1.copydimsComplexArray(b);
			Tcomplex *c1p=c1.ComplexPtr(); Tint *bp=b.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
			c = c1;
		} else {
			c.copydimsComplexArray(b);
			Tint *bp=b.IntPtr();
			Tcomplex *cp=c.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
		}
		gl_nops+= N;
		break;
	case pair(Kcomplex,KRealArray):
		N = b.dims().length();
		Z = a.z;
		if (&c == &b) {
			Tobject c1;
			c1.copydimsComplexArray(b);
			Tcomplex *c1p=c1.ComplexPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
			c = c1;
		} else {
			c.copydimsComplexArray(b);
			Tcomplex *cp=c.ComplexPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
		}
		gl_nops+= N;
		break;
	case pair(Kcomplex,KComplexArray):
		{
			c.copydimsComplexArray(b);
			N = b.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			Z = a.z;
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kint):
		{
			c.copydimsIntArray(a);
			I = b.i;
			N = a.dims().length();
			Tint *cp=c.IntPtr(), *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],I);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kreal):
		N = a.dims().length();
		X = b.x;
		if (&c == &a) {
			Tobject c1;
			c1.copydimsRealArray(a);
			Treal *c1p=c1.RealPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],X);
			c = c1;
		} else {
			c.copydimsRealArray(a);
			Treal *cp=c.RealPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
		}
		gl_nops+= N;
		break;
	case pair(KIntArray,Kcomplex):
		N = a.dims().length();
		Z = b.z;
		if (&c == &a) {
			Tobject c1;
			c1.copydimsComplexArray(a);
			Tcomplex *c1p=c1.ComplexPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
			c = c1;
		} else {
			c.copydimsComplexArray(a);
			Tcomplex *cp=c.ComplexPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
		}
		gl_nops+= N;
		break;
	case pair(KIntArray,KIntArray):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *cp=c.IntPtr(), *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KRealArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *bp=b.RealPtr(); Tint *ap=a.IntPtr();
			if (&a == &c) {
				Tobject c1;
				c1.copydimsRealArray(b);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(Treal(ap[i]),bp[i]);
				c = c1;
			} else {
				c.copydimsRealArray(b);
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KComplexArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *bp=b.ComplexPtr(); Tint *ap=a.IntPtr();
			if (&a == &c) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kint):
		{
			c.copydimsRealArray(a);
			X = Treal(b.i);
			N = a.dims().length();
			Treal *cp=c.RealPtr(), *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kreal):
		{
			c.copydimsRealArray(a);
			N = a.dims().length();
			Treal *cp=c.RealPtr(), *ap=a.RealPtr();
			X = b.x;
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kcomplex):
		N = a.dims().length();
		if (&c == &a) {
			Tobject c1;
			c1.copydimsComplexArray(a);
			Tcomplex *c1p=c1.ComplexPtr(); Treal *ap=a.RealPtr();
			Z = b.z;
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
			c = c1;
		} else {
			c.copydimsComplexArray(a);
			Tcomplex *cp=c.ComplexPtr(); Treal *ap=a.RealPtr();
			Z = b.z;
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
		}
		gl_nops+= N;
		break;
	case pair(KRealArray,KIntArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *ap=a.RealPtr(); Tint *bp=b.IntPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsRealArray(a);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],Treal(bp[i]));
				c = c1;
			} else {
				c.copydimsRealArray(a);
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],Treal(bp[i]));
			}
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KRealArray):
		{
			c.copydimsRealArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *cp=c.RealPtr(), *ap=a.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KComplexArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *bp=b.ComplexPtr(); Treal *ap=a.RealPtr();
			if (&a == &c) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kint):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			Z = Tcomplex(b.i);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kreal):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			Z = Tcomplex(b.x);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kcomplex):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			Z = b.z;
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KIntArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *ap=a.ComplexPtr(); Tint *bp=b.IntPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KRealArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *ap=a.ComplexPtr(); Treal *bp=b.RealPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KComplexArray):
		{
			c.copydimsComplexArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *cp=c.RealPtr(), *bp=b.RealPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),bp[i]);
			gl_nops+= N;
		}
		break;
	default:
		BinError(OPCHAR,a,b);
		break;
	}
}
