/*
 * This file is part of tela the Tensor Language.
 * Copyright (c) 1994-1995 Pekka Janhunen
 */

/*
 * tbinrealop.C - 'template' for a real binary operator, included for "/"
 * NOTE!! Currently this is not used but Div is hardcoded in object.C.
 */

void OPFNAME(Tobject& c, const Tobject& a, const Tobject& b) {
	int i,N;
	Treal X; Tcomplex Z;
	switch (pair(a.kind(),b.kind())) {
	case pair(Kint,Kint):
		c = OP(Treal(a.IntValue()),Treal(b.IntValue()));
		global::nops++;
		break;
	case pair(Kint,Kreal):
		c = OP(Treal(a.IntValue()),b.RealValue());
		global::nops++;
		break;
	case pair(Kint,Kcomplex):
		c = OP(Tcomplex(a.IntValue()),b.ComplexValue());
		global::nops++;
		break;
	case pair(Kint,KIntArray):
		{
			X = Treal(a.IntValue());
			N = b.length();
			Tint *bp = b.IntPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsRealArray(b);
				Treal *c1p = c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(X,Treal(bp[i]));
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsRealArray(b);
				Treal *cp = c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(X,Treal(bp[i]));
			}
			global::nops+= N;
		}
		break;
	case pair(Kint,KRealArray):
		{
			X = Treal(a.IntValue());
			N = b.length();
			c.copydimsRealArray(b);
			Treal *cp=c.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			global::nops+= N;
		}
		break;
	case pair(Kint,KComplexArray):
		{
			Z = Tcomplex(Treal(a.IntValue()),0.0);
			N = b.length();
			c.copydimsComplexArray(b);
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			global::nops+= N;
		}
		break;
	case pair(Kreal,Kint):
		c = OP(a.RealValue(),Treal(b.IntValue()));
		global::nops++;
		break;
	case pair(Kreal,Kreal):
		c = OP(a.RealValue(),b.RealValue());
		global::nops++;
		break;
	case pair(Kreal,Kcomplex):
		c = OP(Tcomplex(a.RealValue()),b.ComplexValue());
		global::nops++;
		break;
	case pair(Kreal,KIntArray):
		{
			N = b.length();
			X = a.RealValue();
			Tint *bp=b.IntPtr();
			if (&c == &b) {
				Tobject c1;
				c1.copydimsRealArray(b);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(X,bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsRealArray(b);	// make c a real array of the same dims as b
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(Kreal,KRealArray):
		{
			N = b.length();
			X = a.RealValue();
			c.copydimsRealArray(b);
			Treal *cp=c.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			global::nops+= N;
		}
		break;
	case pair(Kreal,KComplexArray):
		{
			Z = a.RealValue();
			N = b.length();
			c.copydimsComplexArray(b);
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			global::nops+= N;
		}
		break;
	case pair(Kcomplex,Kint):
		c = OP(a.ComplexValue(),Tcomplex(b.IntValue()));
		global::nops++;
		break;
	case pair(Kcomplex,Kreal):
		c = OP(a.ComplexValue(),Tcomplex(b.RealValue()));
		global::nops++;
		break;
	case pair(Kcomplex,Kcomplex):
		c = OP(a.ComplexValue(),b.ComplexValue());
		global::nops++;
		break;
	case pair(Kcomplex,KIntArray):
		{
			N = b.length();
			Z = a.ComplexValue();
			Tint *bp=b.IntPtr();
			if (&c == &b) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(Kcomplex,KRealArray):
		{
			N = b.length();
			Z = a.ComplexValue();
			Treal *bp=b.RealPtr();
			if (&c == &b) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(Kcomplex,KComplexArray):
		{
			N = b.length();
			Z = a.ComplexValue();
			c.copydimsComplexArray(b);
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			global::nops+= N;
		}
		break;
	case pair(KIntArray,Kint):
		{
			X = Treal(b.IntValue());
			N = a.length();
			c.copydimsRealArray(a);
			Tint *ap=a.IntPtr();
			if (&a == &c) {
				Tobject c1;
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],X);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			}
			global::nops+= N;
		}
		break;
	case pair(KIntArray,Kreal):
		{
			N = a.length();
			X = b.RealValue();
			Tint *ap=a.IntPtr();
			if (&c == &a) {
				Tobject c1;
				c1.copydimsRealArray(a);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],X);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsRealArray(a);
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			}
			global::nops+= N;
		}
		break;
	case pair(KIntArray,Kcomplex):
		{
			N = a.length();
			Z = b.ComplexValue();
			Tint *ap=a.IntPtr();
			if (&c == &a) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			}
			global::nops+= N;
		}
		break;
	case pair(KIntArray,KIntArray):
		N = a.length();
		//if (N != b.length()) LengthError(OPCHAR,a,b);
		if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
		if (&c == &a || &c == &b) {
			Tobject c1;
			c1.copydimsRealArray(a);
			Treal *c1p=c1.RealPtr(); Tint *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Treal(ap[i]),Treal(bp[i]));
			//c = c1;
			c.bitwiseMoveFrom(c1);
		} else {
			c.copydimsRealArray(a);
			Treal *cp=c.RealPtr(); Tint *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),Treal(bp[i]));
		}
		global::nops+= N;
		break;
	case pair(KIntArray,KRealArray):
		{
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			if (&a == &c) {
				Tobject c1;
				c1.copydimsRealArray(b);
				Treal *cp=c1.RealPtr(), *bp=b.RealPtr(); Tint *ap=a.IntPtr();
				for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsRealArray(b);
				Treal *cp=c.RealPtr(), *bp=b.RealPtr(); Tint *ap=a.IntPtr();
				for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(KIntArray,KComplexArray):
		{
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Tint *ap=a.IntPtr(); Tcomplex *bp=b.ComplexPtr();
			if (&a == &c) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(KRealArray,Kint):
		{
			X = Treal(b.IntValue());
			N = a.length();
			c.copydimsRealArray(a);
			Treal *cp=c.RealPtr(), *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			global::nops+= N;
		}
		break;
	case pair(KRealArray,Kreal):
		{
			N = a.length();
			X = b.RealValue();
			c.copydimsRealArray(a);
			Treal *cp=c.RealPtr(), *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			global::nops+= N;
		}
		break;
	case pair(KRealArray,Kcomplex):
		{
			N = a.length();
			Treal *ap=a.RealPtr();
			Z = b.ComplexValue();
			if (&c == &a) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			}
			global::nops+= N;
		}
		break;
	case pair(KRealArray,KIntArray):
		{
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Tint *bp=b.IntPtr();
			Treal *ap=a.RealPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsRealArray(a);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],Treal(bp[i]));
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsRealArray(a);
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],Treal(bp[i]));
			}
			global::nops+= N;
		}
		break;
	case pair(KRealArray,KRealArray):
		{
			c.copydimsRealArray(a);
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Treal *cp=c.RealPtr(), *ap=a.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			global::nops+= N;
		}
		break;
	case pair(KRealArray,KComplexArray):
		{
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Treal *ap=a.RealPtr(); Tcomplex *bp=b.ComplexPtr();
			if (&a == &c) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(KComplexArray,Kint):
		{
			N = a.length();
			Z = Tcomplex(b.IntValue());
			c.copydimsComplexArray(a);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			global::nops+= N;
		}
		break;
	case pair(KComplexArray,Kreal):
		{
			N = a.length();
			Z = Tcomplex(b.RealValue());
			c.copydimsComplexArray(a);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			global::nops+= N;
		}
		break;
	case pair(KComplexArray,Kcomplex):
		{
			N = a.length();
			Z = b.ComplexValue();
			c.copydimsComplexArray(a);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			global::nops+= N;
		}
		break;
	case pair(KComplexArray,KIntArray):
		{
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Tcomplex *ap=a.ComplexPtr(); Tint *bp=b.IntPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(KComplexArray,KRealArray):
		{
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Tcomplex *ap=a.ComplexPtr(); Treal *bp=b.RealPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				//c = c1;
				c.bitwiseMoveFrom(c1);
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			global::nops+= N;
		}
		break;
	case pair(KComplexArray,KComplexArray):
		{
			c.copydimsComplexArray(b);
			N = a.length();
			//if (N != b.length()) LengthError(OPCHAR,a,b);
			if (!a.iscompatible(b)) LengthError(OPCHAR,a,b);
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			global::nops+= N;
		}
		break;
	default:
		BinError(OPCHAR,a,b);
		break;
	}
}
