/*
 * tcmpop.C - 'template' for a binary compare operator, included separately for >,<,>=,<=,==,!=
 */

void OPFNAME(Tobject& c, const Tobject& a, const Tobject& b) {
	int i,N;
	Tint I; Treal X;
	switch (pair(a.kind(),b.kind())) {
	case pair(Kint,Kint):
		c = OP(a.i,b.i);
		gl_nops++;
		break;
	case pair(Kint,Kreal):
		c = OP(Treal(a.i),b.x);
		gl_nops++;
		break;
	case pair(Kint,KIntArray):
		{
			c.copydimsIntArray(b);
			I = a.i;
			N = b.dims().length();
			Tint *cp=c.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(I,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kint,KRealArray):
		{
			c.copydimsIntArray(b);
			N = b.dims().length();
			X = Treal(a.i);
			Tint *cp=c.IntPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,Kint):
		c = OP(a.x,Treal(b.i));
		gl_nops++;
		break;
	case pair(Kreal,Kreal):
		c = OP(a.x,b.x);
		gl_nops++;
		break;
	case pair(Kreal,KIntArray):
		{
			c.copydimsIntArray(b);	// make c an int array of the same dims as b
			N = b.dims().length();
			Tint *cp=c.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(a.x,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,KRealArray):
		{
			c.copydimsIntArray(b);
			N = b.dims().length();
			Tint *cp=c.IntPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(a.x,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kint):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			I = b.i;
			Tint *cp=c.IntPtr(), *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],I);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kreal):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			Tint *cp=c.IntPtr(), *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],b.x);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KIntArray):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *cp=c.IntPtr(), *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KRealArray):
		{
			c.copydimsIntArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *cp=c.IntPtr(), *ap=a.IntPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kint):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			X = Treal(b.i);
			Tint *cp=c.IntPtr(); Treal *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kreal):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			Tint *cp=c.IntPtr(); Treal *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],b.x);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KIntArray):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *cp=c.IntPtr(), *bp=b.IntPtr(); Treal *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Treal(bp[i]));
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KRealArray):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *cp=c.IntPtr(); Treal *ap=a.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	default:
		BinError(OPCHAR,a,b);
		break;
	}
}
