/*
 * teqop.C - 'template' for an equality operator, included separately for ==, !=
 */

void OPFNAME(Tobject& c, const Tobject& a, const Tobject& b) {
	int i,N;
	Tint I; Treal X; Tcomplex Z;
	switch (pair(a.kind(),b.kind())) {
	case pair(Kint,Kint):
		c = OP(a.i,b.i);
		gl_nops++;
		break;
	case pair(Kint,Kreal):
		c = OP(Treal(a.i),b.x);
		gl_nops++;
		break;
	case pair(Kint,Kcomplex):
		c = OP(Tcomplex(a.i),b.z);
		gl_nops++;
		break;
	case pair(Kint,KIntArray):
		{
			c.copydimsIntArray(b);
			I = a.i;
			N = b.dims().length();
			Tint *cp=c.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(I,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kint,KRealArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			X = Treal(a.i);
			N = b.dims().length();
			Tint *c1p=c1.IntPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(X,bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(Kint,KComplexArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			Z = Tcomplex(Treal(a.i),0.0);
			N = b.dims().length();
			Tint *c1p=c1.IntPtr(); Tcomplex *bp=b.ComplexPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(Kreal,Kint):
		c = OP(a.x,Treal(b.i));
		gl_nops++;
		break;
	case pair(Kreal,Kreal):
		c = OP(a.x,b.x);
		gl_nops++;
		break;
	case pair(Kreal,Kcomplex):
		c = OP(Tcomplex(a.x),b.z);
		gl_nops++;
		break;
	case pair(Kreal,KIntArray):
		{
			N = b.dims().length();
			c.copydimsIntArray(b);	// make c an int array of the same dims as b
			Tint *cp=c.IntPtr(), *bp=b.IntPtr();
			X = a.x;
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,KRealArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			N = b.dims().length();
			X = a.x;
			Tint *c1p=c1.IntPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(X,bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(Kreal,KComplexArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			Z = a.x;
			N = b.dims().length();
			Tint *c1p=c1.IntPtr(); Tcomplex *bp=b.ComplexPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,Kint):
		c = OP(a.z,Tcomplex(b.i));
		gl_nops++;
		break;
	case pair(Kcomplex,Kreal):
		c = OP(a.z,Tcomplex(b.x));
		gl_nops++;
		break;
	case pair(Kcomplex,Kcomplex):
		c = OP(a.z,b.z);
		gl_nops++;
		break;
	case pair(Kcomplex,KIntArray):
		{
			N = b.dims().length();
			c.copydimsIntArray(b);
			Z = a.z;
			Tint *cp=c.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,KRealArray):
		{
			N = b.dims().length();
			Tobject c1;
			c1.copydimsIntArray(b);
			Tint *c1p=c1.IntPtr(); Treal *bp=b.RealPtr();
			Z = a.z;
			for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,KComplexArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			N = b.dims().length();
			Tint *c1p=c1.IntPtr(); Tcomplex *bp=b.ComplexPtr();
			Z = a.z;
			for (i=0; i<N; i++) c1p[i] = OP(Z,bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kint):
		{
			c.copydimsIntArray(a);
			I = b.i;
			N = a.dims().length();
			Tint *cp=c.IntPtr(), *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],I);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kreal):
		{
			N = a.dims().length();
			c.copydimsIntArray(a);
			X = b.x;
			Tint *cp=c.IntPtr(), *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kcomplex):
		{
			N = a.dims().length();
			c.copydimsIntArray(a);
			Tint *cp=c.IntPtr(), *ap=a.IntPtr();
			Z = b.z;
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KIntArray):
		{
			c.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *cp=c.IntPtr(), *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KRealArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Treal *bp=b.RealPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Treal(ap[i]),bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KComplexArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Tcomplex *bp=b.ComplexPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kint):
		{
			Tobject c1;
			c1.copydimsRealArray(a);
			X = Treal(b.i);
			N = a.dims().length();
			Tint *c1p=c1.IntPtr(); Treal *ap=a.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],X);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kreal):
		{
			Tobject c1;
			c1.copydimsRealArray(a);
			N = a.dims().length();
			Tint *c1p=c1.IntPtr(); Treal *ap=a.RealPtr();
			X = b.x;
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],X);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kcomplex):
		{
			N = a.dims().length();
			Tobject c1;
			c1.copydimsIntArray(a);
			Tint *c1p=c1.IntPtr(); Treal *ap=a.RealPtr();
			Z = b.z;
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KIntArray):
		{
			Tobject c1;
			c1.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Treal *ap=a.RealPtr(); Tint *bp=b.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Treal(bp[i]));
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KRealArray):
		{
			Tobject c1;
			c1.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Treal *ap=a.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KComplexArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Tcomplex *bp=b.ComplexPtr(); Treal *ap=a.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kint):
		{
			Tobject c1;
			c1.copydimsComplexArray(a);
			N = a.dims().length();
			Z = Tcomplex(b.i);
			Tint *c1p=c1.IntPtr(); Tcomplex *ap=a.ComplexPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kreal):
		{
			Tobject c1;
			c1.copydimsIntArray(a);
			N = a.dims().length();
			Z = Tcomplex(b.x);
			Tint *c1p=c1.IntPtr(); Tcomplex *ap=a.ComplexPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kcomplex):
		{
			Tobject c1;
			c1.copydimsIntArray(a);
			N = a.dims().length();
			Tint *c1p=c1.IntPtr(); Tcomplex *ap=a.ComplexPtr();
			Z = b.z;
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],Z);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KIntArray):
		{
			Tobject c1;
			c1.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Tcomplex *ap=a.ComplexPtr(); Tint *bp=b.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KRealArray):
		{
			Tobject c1;
			c1.copydimsIntArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Tcomplex *ap=a.ComplexPtr(); Treal *bp=b.RealPtr();
			for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KComplexArray):
		{
			Tobject c1;
			c1.copydimsIntArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *c1p=c1.IntPtr(); Treal *bp=b.RealPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Treal(ap[i]),bp[i]);
			c = c1;
			gl_nops+= N;
		}
		break;
	default:
		BinError(OPCHAR,a,b);
		break;
	}
}
